""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/11/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians
        : based on Mattia Bulla's PhD thesis

History : (02/09/2013) First version.

"""


import numpy as np


def MuellerThomsonRayleighScatteringMatrix (angle=np.radians(0.0)):
    #
    fcp = (np.cos(angle))**2+1
    fcm = (np.cos(angle))**2-1
    fc2 = 2*np.cos(angle)
    #
    r1 = [fcp, fcm, 0.0, 0.0]
    r2 = [fcm, fcp, 0.0, 0.0]
    r3 = [0.0, 0.0, fc2, 0.0]
    r4 = [0.0, 0.0, 0.0, fc2]
    return 0.75**np.matrix([r1, r2, r3, r4])



""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 23/07/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (06/05/2013) First version.
        : (23/07/2015) python3 comparison.
"""

import os


from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPCatalogue.GetSDSS import GetSDSS


class SDSS:
    # SDSS entries
    class CatEntries:
        def __init__ (self,id,ra,dec,u,eu,g,eg,r,er,i,ei,z,ez):
            self.Id = id
            self.RA = float(ra)
            self.DEC = float(dec)
            inpmag = AstroMagInput (u,eu)
            self.u = inpmag.Mag
            self.eu =inpmag.eMag
            inpmag = AstroMagInput (g,eg)
            self.g = inpmag.Mag
            self.eg =inpmag.eMag
            inpmag = AstroMagInput (r,er)
            self.r = inpmag.Mag
            self.er =inpmag.eMag
            inpmag = AstroMagInput (i,ei)
            self.i = inpmag.Mag
            self.ei =inpmag.eMag
            inpmag = AstroMagInput (z,ez)
            self.z = inpmag.Mag
            self.ez =inpmag.eMag

        
        def __str__ (self):
            msg = "%30s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.u, self.eu)
            msg = msg + "%10.3f\t%9.3f\t" % (self.g, self.eg)
            msg = msg + "%10.3f\t%9.3f" % (self.r, self.er)
            msg = msg + "%10.3f\t%9.3f" % (self.i, self.ei)
            msg = msg + "%10.3f\t%9.3f" % (self.z, self.ez)
            return msg
            
 
        def __lt__ (self, other):
            return self.r < other.r
 
 
 
    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Epoch = 2000.0
        self.Radius = radius
        self.ListEntries = []
        
        
    def GetData(self):
        sdss = GetSDSS (self.RA,self.DEC,self.Radius)
        if len(sdss) == 0:
            return None
        for il in sdss:
            id = il['SDSS12']
            ra = il['RA_ICRS']
            dec = il['DE_ICRS']
            coord = AstroCoordInput(ra,dec)
            u = il['umag']
            eu = il['e_umag']
            g = il['gmag']
            eg = il['e_gmag']
            r = il['rmag']
            er = il['e_rmag']
            i = il['imag']
            ei = il['e_imag']
            z = il['zmag']
            ez = il['e_zmag']
            newinp = self.CatEntries(id,coord.RA,coord.DEC,u,eu,g,eg,r,er,i,ei,z,ez)
            # No duplicate entries
            for ii in self.ListEntries:
                if newinp.Id == ii.Id:
                    break
            else:
                self.ListEntries.append(newinp)
        return len(self.ListEntries)

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg



    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tu\teu\tg\teg\tr\ter\ti\tei\tz\tez\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg

        
        
    def sort(self):
        self.ListEntries.sort()





""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.4
Author  : Stefano Covino
Date    : 04/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians, real and imaginary part of the refraction index
            can be found at: http://refractiveindex.info/
            From Joos et al., SPIE, 7016
            

History : (04/02/2012) First version.
        : (07/07/2016) Correction for the delta expression.
		: (09/11/2016) Using numpy only.
        : (14/02/2017) Minor bug.
        : (04/04/2017) Minor rewrite.
"""

import numpy as np


def MuellerMetallicMirrorMatrix (refrindex=1.26, extcoeff=7.19, angle=np.radians(45.0)):
    p = refrindex**2-extcoeff**2-np.sin(angle)**2
    q = 2*refrindex*extcoeff
    #
    rpiu = (1./np.sqrt(2.))*np.sqrt(p+np.sqrt(p**2+q**2))
    rmeno = (1./np.sqrt(2.))*np.sqrt(-p+np.sqrt(p**2+q**2))
    s = np.sin(angle)*np.tan(angle)
    #
    rho = np.sqrt((np.sqrt(p**2+q**2)+s**2-2*s*rpiu)/
            (np.sqrt(p**2+q**2)+s**2+2*s*rpiu))
    delta = np.arctan((2*s*rmeno)/(np.sqrt(p**2+q**2)-s**2))
    #
    r1 = [1.+rho**2, 1-rho**2, 0., 0.]
    r2 = [1.-rho**2, 1.+rho**2, 0., 0.]
    r3 = [0., 0., -2.*rho*np.cos(delta), -2.*rho*np.sin(delta)]
    r4 = [0., 0., 2.*rho*np.sin(delta), -2.*rho*np.cos(delta)]
    return 0.5*np.matrix([r1, r2, r3, r4])



""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 19/05/2021
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (19/05/2021) First version.

"""


import os, os.path


def ReadTextFile(filename):
    try:
        with open(filename) as f:
            lst=[line.strip() for line in f]
        return lst
    except FileNotFoundError:
        return None




""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 18/09/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angles are in radians

History : (18/09/2017) First version.
"""

import numpy as np


def Cnt2Stoke (co,eco,ce,ece):
    s = (co-ce)/(co+ce)
    es = abs(s) * np.sqrt( ((eco**2+ece**2)/(co-ce)**2) + ((eco**2+ece**2)/(co+ce)**2) )
    return s,es



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 28/01/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (28/01/2016) First version.

"""


import numpy



def findPercAreaBis (x,y,r,xp,yp):
    intersezioni = []
    xA = xp - 1
    sqr = r**2-(xA-x)**2
    if sqr >= 0:
        yA1 = y + math.sqrt(sqr)
        yA2 = y - math.sqrt(sqr)
        if yp-1 <= yA1 <= yp:
            intersezioni.append((xA,yA1,True,1))
        if yp-1 <= yA2 <= yp:
            intersezioni.append((xA,yA2,False,1))
    xB = xp
    sqr = r**2-(xB-x)**2
    if sqr >= 0:
        yB1 = y + math.sqrt(sqr)
        yB2 = y - math.sqrt(sqr)
        if yp-1 <= yB1 <= yp:
            intersezioni.append((xB,yB1,True,2))
        if yp-1 <= yB2 <= yp:
            intersezioni.append((xB,yB2,False,2))
    yC = yp - 1
    sqr = r**2-(yC-y)**2
    if sqr >= 0.0:
        xC1 = x + math.sqrt(sqr)
        xC2 = x - math.sqrt(sqr)
        if xp-1 <= xC1 <= xp:
            intersezioni.append((xC1,yC,True,3))
        if xp-1 <= xC2 <= xp:
            intersezioni.append((xC2,yC,False,3))
    yD = yp
    sqr = r**2-(yD-y)**2
    if sqr >= 0.0:
        xD1 = x + math.sqrt(sqr)
        xD2 = x - math.sqrt(sqr)
        if xp-1 <= xD1 <= xp:
            intersezioni.append((xD1,yD,True,4))
        if xp-1 <= xD2 <= xp:
            intersezioni.append((xD2,yD,False,4))
    if len(intersezioni) == 2:
        if yp-1 >= y or yp <= y:
            if intersezioni[0][0] <= intersezioni[1][0]:
                area1 = (intersezioni[0][0]-(xp-1))*(intersezioni[0][1]-(yp-1))
                area3 = (xp-intersezioni[1][0])*(intersezioni[1][1]-(yp-1))
                if yp-1 >= y:
                    area2 = integFuct(intersezioni[0][0],intersezioni[1][0],x,y,r,yp)
                elif yp <= y:
                    area2 = integFuct(intersezioni[0][0],intersezioni[1][0],x,y,r,yp,False)
            else:
                area1 = (intersezioni[1][0]-(xp-1))*(intersezioni[1][1]-(yp-1))
                area3 = (xp-intersezioni[0][0])*(intersezioni[0][1]-(yp-1))
                if yp-1 >= y:
                    area2 = integFuct(intersezioni[1][0],intersezioni[0][0],x,y,r,yp)
                elif yp <= y:
                    area2 = integFuct(intersezioni[1][0],intersezioni[0][0],x,y,r,yp,False)
            area = area1 + area2 + area3
        else:
            if xp-1 >= x:
                if intersezioni[0][1] <= intersezioni[1][1]:
                    area = integFuct(intersezioni[0][1],intersezioni[1][1],y,x,r,xp)
                else:
                    area = integFuct(intersezioni[1][1],intersezioni[0][1],y,x,r,xp)
            else:
                if intersezioni[0][1] <= intersezioni[1][1]:
                    area = integFuct(intersezioni[0][1],intersezioni[1][1],y,x,r,xp,False)
                else:
                    area = integFuct(intersezioni[1][1],intersezioni[0][1],y,x,r,xp,False)
        if yp-1 >= y:
            return area
        elif yp <= y:
            return 1.0-area
        else:
            if xp-1 >= x:
                return area
            elif xp <= x:
                return 1-area
    else:
        dist = math.sqrt((x-xp)**2+(y-yp)**2)
        if dist < r:
            return 1.0
        else:
            return 0.0





""" Init file for SRPPolarimetry

Context : SRP
Module  : Polarimetry
Version : 1.2.0
Author  : Stefano Covino
Date    : 18/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (15/02/2012) First named version.
        : (10/11/2015) resid, centerGauss and getBackground added.
        : (13/11/2015) MinMax added.
        : (18/11/2015) centerMoment added.
"""


__all__ = ['centerGauss', 'centerMoment', 'Counts2Mag', 'Mag2Counts', 'getBackground',
            'MinMax', 'resid']






#!/usr/bin/python
# -*- coding: iso-8859-1 -*-
"""
HTML.py - v0.04 2009-07-28 Philippe Lagadec

This module provides a few classes to easily generate HTML code such as tables
and lists.

Project website: http://www.decalage.info/python/html

License: CeCILL (open-source GPL compatible), see source code for details.
         http://www.cecill.info
"""

__version__ = '0.04'
__date__    = '2009-07-28'
__author__  = 'Philippe Lagadec'

#--- LICENSE ------------------------------------------------------------------

# Copyright Philippe Lagadec - see http://www.decalage.info/contact for contact info
#
# This module provides a few classes to easily generate HTML tables and lists.
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# A copy of the CeCILL license is also provided in these attached files:
# Licence_CeCILL_V2-en.html and Licence_CeCILL_V2-fr.html
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


#--- CHANGES ------------------------------------------------------------------

# 2008-10-06 v0.01 PL: - First version
# 2008-10-13 v0.02 PL: - added cellspacing and cellpadding to table
#                      - added functions to ease one-step creation of tables
#                        and lists
# 2009-07-21 v0.03 PL: - added column attributes and styles (first attempt)
#                        (thanks to an idea submitted by Michal Cernoevic)
# 2009-07-28 v0.04 PL: - improved column styles, workaround for Mozilla


#-------------------------------------------------------------------------------
#TODO:
# - method to return a generator (yield each row) instead of a single string
# - unicode support (input and output)
# - escape text in cells (optional)
# - constants for standard colors
# - use lxml to generate well-formed HTML ?
# - add classes/functions to generate a HTML page, paragraphs, headings, etc...


#--- THANKS --------------------------------------------------------------------

# - Michal Cernoevic, for the idea of column styles.

#--- REFERENCES ----------------------------------------------------------------

# HTML 4.01 specs: http://www.w3.org/TR/html4/struct/tables.html

# Colors: http://www.w3.org/TR/html4/types.html#type-color

# Columns alignement and style, one of the oldest and trickiest bugs in Mozilla:
# https://bugzilla.mozilla.org/show_bug.cgi?id=915


#--- CONSTANTS -----------------------------------------------------------------

# Table style to get thin black lines in Mozilla/Firefox instead of 3D borders
TABLE_STYLE_THINBORDER = "border: 1px solid #000000; border-collapse: collapse;"
#TABLE_STYLE_THINBORDER = "border: 1px solid #000000;"


#=== CLASSES ===================================================================

class TableCell (object):
    """
    a TableCell object is used to create a cell in a HTML table. (TD or TH)

    Attributes:
    - text: text in the cell (may contain HTML tags). May be any object which
            can be converted to a string using str().
    - header: bool, false for a normal data cell (TD), true for a header cell (TH)
    - bgcolor: str, background color
    - width: str, width
    - align: str, horizontal alignement (left, center, right, justify or char)
    - char: str, alignment character, decimal point if not specified
    - charoff: str, see HTML specs
    - valign: str, vertical alignment (top|middle|bottom|baseline)
    - style: str, CSS style
    - attribs: dict, additional attributes for the TD/TH tag

    Reference: http://www.w3.org/TR/html4/struct/tables.html#h-11.2.6
    """

    def __init__(self, text="", bgcolor=None, header=False, width=None,
                align=None, char=None, charoff=None, valign=None, style=None,
                attribs=None):
        """TableCell constructor"""
        self.text    = text
        self.bgcolor = bgcolor
        self.header  = header
        self.width   = width
        self.align   = align
        self.char    = char
        self.charoff = charoff
        self.valign  = valign
        self.style   = style
        self.attribs = attribs
        if attribs==None:
            self.attribs = {}

    def __str__(self):
        """return the HTML code for the table cell as a string"""
        attribs_str = ""
        if self.bgcolor: self.attribs['bgcolor'] = self.bgcolor
        if self.width:   self.attribs['width']   = self.width
        if self.align:   self.attribs['align']   = self.align
        if self.char:    self.attribs['char']    = self.char
        if self.charoff: self.attribs['charoff'] = self.charoff
        if self.valign:  self.attribs['valign']  = self.valign
        if self.style:   self.attribs['style']   = self.style
        for attr in self.attribs:
            attribs_str += ' %s="%s"' % (attr, self.attribs[attr])
        if self.text:
            text = str(self.text)
        else:
            # An empty cell should at least contain a non-breaking space
            text = '&nbsp;'
        if self.header:
            return '  <TH%s>%s</TH>\n' % (attribs_str, text)
        else:
            return '  <TD%s>%s</TD>\n' % (attribs_str, text)

#-------------------------------------------------------------------------------

class TableRow (object):
    """
    a TableRow object is used to create a row in a HTML table. (TR tag)

    Attributes:
    - cells: list, tuple or any iterable, containing one string or TableCell
             object for each cell
    - header: bool, true for a header row (TH), false for a normal data row (TD)
    - bgcolor: str, background color
    - col_align, col_valign, col_char, col_charoff, col_styles: see Table class
    - attribs: dict, additional attributes for the TR tag

    Reference: http://www.w3.org/TR/html4/struct/tables.html#h-11.2.5
    """

    def __init__(self, cells=None, bgcolor=None, header=False, attribs=None,
                col_align=None, col_valign=None, col_char=None,
                col_charoff=None, col_styles=None):
        """TableCell constructor"""
        self.bgcolor     = bgcolor
        self.cells       = cells
        self.header      = header
        self.col_align   = col_align
        self.col_valign  = col_valign
        self.col_char    = col_char
        self.col_charoff = col_charoff
        self.col_styles  = col_styles
        self.attribs     = attribs
        if attribs==None:
            self.attribs = {}

    def __str__(self):
        """return the HTML code for the table row as a string"""
        attribs_str = ""
        if self.bgcolor: self.attribs['bgcolor'] = self.bgcolor
        for attr in self.attribs:
            attribs_str += ' %s="%s"' % (attr, self.attribs[attr])
        result = ' <TR%s>\n' % attribs_str
        for cell in self.cells:
            col = self.cells.index(cell)    # cell column index
            if not isinstance(cell, TableCell):
                cell = TableCell(cell, header=self.header)
            # apply column alignment if specified:
            if self.col_align and cell.align==None:
                cell.align = self.col_align[col]
            if self.col_char and cell.char==None:
                cell.char = self.col_char[col]
            if self.col_charoff and cell.charoff==None:
                cell.charoff = self.col_charoff[col]
            if self.col_valign and cell.valign==None:
                cell.valign = self.col_valign[col]
            # apply column style if specified:
            if self.col_styles and cell.style==None:
                cell.style = self.col_styles[col]
            result += str(cell)
        result += ' </TR>\n'
        return result

#-------------------------------------------------------------------------------

class Table (object):
    """
    a Table object is used to create a HTML table. (TABLE tag)

    Attributes:
    - rows: list, tuple or any iterable, containing one iterable or TableRow
            object for each row
    - header_row: list, tuple or any iterable, containing the header row (optional)
    - border: str or int, border width
    - style: str, table style in CSS syntax (thin black borders by default)
    - width: str, width of the table on the page
    - attribs: dict, additional attributes for the TABLE tag
    - col_width: list or tuple defining width for each column
    - col_align: list or tuple defining horizontal alignment for each column
    - col_char: list or tuple defining alignment character for each column
    - col_charoff: list or tuple defining charoff attribute for each column
    - col_valign: list or tuple defining vertical alignment for each column
    - col_styles: list or tuple of HTML styles for each column

    Reference: http://www.w3.org/TR/html4/struct/tables.html#h-11.2.1
    """

    def __init__(self, rows=None, border='1', style=None, width=None,
                cellspacing=None, cellpadding=4, attribs=None, header_row=None,
                col_width=None, col_align=None, col_valign=None,
                col_char=None, col_charoff=None, col_styles=None):
        """TableCell constructor"""
        self.border = border
        self.style = style
        # style for thin borders by default
        if style == None: self.style = TABLE_STYLE_THINBORDER
        self.width       = width
        self.cellspacing = cellspacing
        self.cellpadding = cellpadding
        self.header_row  = header_row
        self.rows        = rows
        if not rows: self.rows = []
        self.attribs     = attribs
        if not attribs: self.attribs = {}
        self.col_width   = col_width
        self.col_align   = col_align
        self.col_char    = col_char
        self.col_charoff = col_charoff
        self.col_valign  = col_valign
        self.col_styles  = col_styles

    def __str__(self):
        """return the HTML code for the table as a string"""
        attribs_str = ""
        if self.border: self.attribs['border'] = self.border
        if self.style:  self.attribs['style'] = self.style
        if self.width:  self.attribs['width'] = self.width
        if self.cellspacing:  self.attribs['cellspacing'] = self.cellspacing
        if self.cellpadding:  self.attribs['cellpadding'] = self.cellpadding
        for attr in self.attribs:
            attribs_str += ' %s="%s"' % (attr, self.attribs[attr])
        result = '<TABLE%s>\n' % attribs_str
        # insert column tags and attributes if specified:
        if self.col_width:
            for width in self.col_width:
                result += '  <COL width="%s">\n' % width
        # The following code would also generate column attributes for style
        # and alignement according to HTML4 specs,
        # BUT it is not supported completely (only width) on Mozilla Firefox:
        # see https://bugzilla.mozilla.org/show_bug.cgi?id=915
##        n_cols = max(len(self.col_styles), len(self.col_width),
##                     len(self.col_align), len(self.col_valign))
##        for i in range(n_cols):
##            col = ''
##            try:
##                if self.col_styles[i]:
##                    col += ' style="%s"' % self.col_styles[i]
##            except: pass
##            try:
##                if self.col_width[i]:
##                    col += ' width="%s"' % self.col_width[i]
##            except: pass
##            try:
##                if self.col_align[i]:
##                    col += ' align="%s"' % self.col_align[i]
##            except: pass
##            try:
##                if self.col_valign[i]:
##                    col += ' valign="%s"' % self.col_valign[i]
##            except: pass
##            result += '<COL%s>\n' % col
        # First insert a header row if specified:
        if self.header_row:
            if not isinstance(self.header_row, TableRow):
                result += str(TableRow(self.header_row, header=True))
            else:
                result += str(self.header_row)
        # Then all data rows:
        for row in self.rows:
            if not isinstance(row, TableRow):
                row = TableRow(row)
            # apply column alignments  and styles to each row if specified:
            # (Mozilla bug workaround)
            if self.col_align and not row.col_align:
                row.col_align = self.col_align
            if self.col_char and not row.col_char:
                row.col_char = self.col_char
            if self.col_charoff and not row.col_charoff:
                row.col_charoff = self.col_charoff
            if self.col_valign and not row.col_valign:
                row.col_valign = self.col_valign
            if self.col_styles and not row.col_styles:
                row.col_styles = self.col_styles
            result += str(row)
        result += '</TABLE>'
        return result


#-------------------------------------------------------------------------------

class List (object):
    """
    a List object is used to create an ordered or unordered list in HTML.
    (UL/OL tag)

    Attributes:
    - lines: list, tuple or any iterable, containing one string for each line
    - ordered: bool, choice between an ordered (OL) or unordered list (UL)
    - attribs: dict, additional attributes for the OL/UL tag

    Reference: http://www.w3.org/TR/html4/struct/lists.html
    """

    def __init__(self, lines=None, ordered=False, start=None, attribs=None):
        """List constructor"""
        if lines:
            self.lines = lines
        else:
            self.lines = []
        self.ordered = ordered
        self.start = start
        if attribs:
            self.attribs = attribs
        else:
            self.attribs = {}

    def __str__(self):
        """return the HTML code for the list as a string"""
        attribs_str = ""
        if self.start:  self.attribs['start'] = self.start
        for attr in self.attribs:
            attribs_str += ' %s="%s"' % (attr, self.attribs[attr])
        if self.ordered: tag = 'OL'
        else:            tag = 'UL'
        result = '<%s%s>\n' % (tag, attribs_str)
        for line in self.lines:
            result += ' <LI>%s\n' % str(line)
        result += '</%s>\n' % tag
        return result


##class Link (object):
##    """
##    a Link object is used to create link in HTML. (<a> tag)
##
##    Attributes:
##    - text: str, text of the link
##    - url: str, URL of the link
##    - attribs: dict, additional attributes for the A tag
##
##    Reference: http://www.w3.org/TR/html4
##    """
##
##    def __init__(self, text, url=None, attribs=None):
##        """Link constructor"""
##        self.text = text
##        self.url = url
##        if attribs:
##            self.attribs = attribs
##        else:
##            self.attribs = {}
##
##    def __str__(self):
##        """return the HTML code for the link as a string"""
##        attribs_str = ""
##        if self.url:  self.attribs['href'] = self.url
##        for attr in self.attribs:
##            attribs_str += ' %s="%s"' % (attr, self.attribs[attr])
##        return '<a%s>%s</a>' % (attribs_str, text)


#=== FUNCTIONS ================================================================

# much simpler definition of a link as a function:
def Link(text, url):
    return '<a href="%s">%s</a>' % (url, text)

def link(text, url):
    return '<a href="%s">%s</a>' % (url, text)

def table(*args, **kwargs):
    'return HTML code for a table as a string. See Table class for parameters.'
    return str(Table(*args, **kwargs))

def list(*args, **kwargs):
    'return HTML code for a list as a string. See List class for parameters.'
    return str(List(*args, **kwargs))


#=== MAIN =====================================================================

# Show sample usage when this file is launched as a script.

if __name__ == '__main__':

    # open an HTML file to show output in a browser
    f = open('test.html', 'w')

    t = Table()
    t.rows.append(TableRow(['A', 'B', 'C'], header=True))
    t.rows.append(TableRow(['D', 'E', 'F']))
    t.rows.append(('i', 'j', 'k'))
    f.write(str(t) + '<p>\n')
    print(str(t))
    print('-'*79)

    t2 = Table([
            ('1', '2'),
            ['3', '4']
        ], width='100%', header_row=('col1', 'col2'),
        col_width=('', '75%'))
    f.write(str(t2) + '<p>\n')
    print(t2)
    print('-'*79)

    t2.rows.append(['5', '6'])
    t2.rows[1][1] = TableCell('new', bgcolor='red')
    t2.rows.append(TableRow(['7', '8'], attribs={'align': 'center'}))
    f.write(str(t2) + '<p>\n')
    print(t2)
    print('-'*79)

    # sample table with column attributes and styles:
    table_data = [
            ['Smith',       'John',         30,    4.5],
            ['Carpenter',   'Jack',         47,    7],
            ['Johnson',     'Paul',         62,    10.55],
        ]
    htmlcode = HTML.table(table_data,
        header_row = ['Last name',   'First name',   'Age', 'Score'],
        col_width=['', '20%', '10%', '10%'],
        col_align=['left', 'center', 'right', 'char'],
        col_styles=['font-size: large', '', 'font-size: small', 'background-color:yellow'])
    f.write(htmlcode + '<p>\n')
    print(htmlcode)
    print('-'*79)

    def gen_table_squares(n):
        """
        Generator to create table rows for integers from 1 to n
        """
##        # First, header row:
##        yield TableRow(('x', 'square(x)'), header=True, bgcolor='blue')
##        # Then all rows:
        for x in range(1, n+1):
            yield (x, x*x)

    t = Table(rows=gen_table_squares(10), header_row=('x', 'square(x)'))
    f.write(str(t) + '<p>\n')

    print('-'*79)
    l = List(['aaa', 'bbb', 'ccc'])
    f.write(str(l) + '<p>\n')
    l.ordered = True
    f.write(str(l) + '<p>\n')
    l.start=10
    f.write(str(l) + '<p>\n')

    f.close()



""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 19/01/2018
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Usage   : PyFind (path, selp='*')
            

History : (02/04/2016) First version.
        : (19/01/2018) Multiple patter managed.
"""


import os, fnmatch



def mpatt (name, mpat):
    if isinstance(mpat,list):
        mm = mpat
    else:
        mm = [mpat]
    #
    for p in mm:
        if fnmatch.fnmatch(name, p):
            return True
    return False



def PyFind (path, selp='*', exclp=[]):
    fileList = []
    #
    for dName, sdName, fList in os.walk(path):
        for fileName in fList:
            if fnmatch.fnmatch(fileName, selp) and not mpatt(fileName, exclp):
                fileList.append(os.path.join(dName, fileName))
    #
    return fileList



""" Init file for SRPCatalogue

Context : SRP
Module  : Catalogue
Version : 1.7.0
Author  : Stefano Covino
Date    : 14/05/2019
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (29/09/2010) First named version.
        : (05/10/2010) AstroWise class.
        : (14/12/2010) NIR collection class.
        : (28/03/2011) AGN optical standard stars.
        : (11/06/2011) USNOB1 class.
        : (15/04/2013) SDSS query script.
        : (06/05/2013) SDSS class.
        : (26/07/2013) APASS class
        : (02/08/2014) SIMBAD name resolver.
        : (29/07/2015) Smith et al. ugriz standard stars.
        : (14/02/2016) SkyBot query.
        : (17/02/2016) IGAIA query.
        : (14/05/2019) SDSS query.
"""



__all__ = ['AGNOptRefStarsClass', 'APASSClass', 'AstroWiseStandardStarsClass', 'BrightStarClass',
           'CatalogueConstants', 'GetSDSS', 'GetVizierCat', 'IGAIAClass', 'NIRStandardStarsClass',
           'ParseVisierCat', 'SDSSClass', 'SimbadNameClass', 'SimbadClass', 'SkyBotClass',
           'SmithetalStandardStarsClass', 'StetsonJohnsonStandardStarsClass', 'TWOMASSClass',
           'USNOB1Class', 'USNOClass']




""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/07/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (29/07/2015) First version.
"""

import os, os.path


import SRP
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPMath.AngularDistance import AngularDistance
from SRP.SRPSystem.SRPPath import SRPPath



class SmithetalStandardStars:
    basepath = SRPPath()
    filename = 'smithetalugrizcat.dat'
    catpath = os.path.join(basepath,'SRPData','Catalogues','SmithetalStandardStars',filename)
    # Catalogue entries
    class CatEntries:
        def __init__ (self, xxx_todo_changeme):
            # Id
            (id,ra,dec,u,eu,g,eg,r,er,i,ei,z,ez) = xxx_todo_changeme
            self.Id = id
            # Coord
            coord = AstroCoordInput(ra,dec)
            self.RA = coord.RA
            self.DEC = coord.DEC
            # u mag
            inpmag = AstroMagInput (u,eu)
            self.u = inpmag.Mag
            self.eu =inpmag.eMag
            # g mag
            inpmag = AstroMagInput (g,eg)
            self.g = inpmag.Mag
            self.eg =inpmag.eMag
            # r mag
            inpmag = AstroMagInput (r,er)
            self.r = inpmag.Mag
            self.er =inpmag.eMag
            # i mag
            inpmag = AstroMagInput (i,ei)
            self.i = inpmag.Mag
            self.ei =inpmag.eMag
            # z mag
            inpmag = AstroMagInput (z,ez)
            self.z = inpmag.Mag
            self.ez =inpmag.eMag

            
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.u, self.eu)
            msg = msg + "%10.3f\t%9.3f\t" % (self.g, self.eg)
            msg = msg + "%10.3f\t%9.3f\t" % (self.r, self.er)
            msg = msg + "%10.3f\t%9.3f\t" % (self.i, self.ei)
            msg = msg + "%10.3f\t%9.3f" % (self.z, self.ez)
            return msg


        def __lt__ (self, other):
            return self.r < other.r
    
    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Radius = radius/60.0
        self.ListEntries = []
        
        
    def GetData(self):
        f = open(self.catpath)
        data = f.readlines()
        f.close()
        if data != None:
            for entry in data:
                parseddata = entry.split()
                newinp = self.CatEntries(parseddata)
                if AngularDistance((self.RA,self.DEC),(newinp.RA,newinp.DEC)) <= self.Radius:
                    self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg


    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tu\teu\tg\teg\tt\tet\ti\tei\tz\tez\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg
        
        
    def sort(self):
        self.ListEntries.sort()




""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 17/02/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Rayleigh sky model according to Harrington et al. (2011, PASP, 123, 799 and
            2016, SPIE,9912,37).
        : cos(γ) = sin(θs)sin(θ)cos(ψ) + cos(θs)cos(θ) where γ is the angular distance 
            between the telescope pointing and the sun, θs is the solar zenith angle, 
            θ is the angular distance between the telescope pointing and the zenith, 
            and ψ is the azimuthal angle between the solar direction and the telescope 
            pointing.

History : (17/02/2017) First version.
"""

import numpy as np


def Gamma (ThetaS, Theta, Phi):
    ts = np.radians(ThetaS)
    t = np.radians(Theta)
    p = np.radians(Phi)
    #
    s = np.sin(ts)*np.sin(t)*np.cos(p)+np.cos(ts)*np.cos(t)
    return np.degrees(np.arcsin(s))


def RayleighSkyModel (gamma, dmax=100.0):
    gm = np.radians(gamma)
    return dmax * (np.sin(gm))**2 / (1 + (np.cos(gm))**2)



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPAstro.py
Version : 1.8.4
Status  : approved
Author  : Stefano Covino
Date    : 16/11/2021
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/~covino
Purpose : Collection of utility functions and classes for SRP.

Usage   : to be imported

Remarks :

History : (19/10/2008) First version.
        : (05/11/2008) Sky in MiPhotdata class.
        : (19/11/2008) Many more minor issues and average sigma-clipping for arrays.
        : (20/11/2008) Better sky estimate and sorting of output file.
        : (27/11/2008) Better center moment estimate.
        : (03/01/2009) Better error estimate.
        : (17/04/2009) Better centering algorithm. MinMax function added.
        : (10/10/2009) Management of negative net fluxes.
        : (12/11/2009) Better (simpler) zero point determination.
        : (25/02/2010) RON in electrons.
        : (21/05/2010) Better sigma clipping algorithm.
        : (17/06/2010) Better managment of objects close to the frame boundaries.
        : (20/06/2010) Better centroid determination.
        : (22/06/2010) Minor improvement.
        : (20/12/2010) Import style corrected.
        : (29/07/2015) numpy porting.
        : (05/11/2015) Minor correction.
        : (10/11/2015) BiGauss, resid and centerGauss improved.
        : (13/11/2015) MinMax improved.
        : (27/11/2015) findPercArea improved.
        : (11/12/2015) Better mangitude computation.
        : (17/12/2015) Returns a limit rather than a detection when the magnitude error is
                        larger than 0.5 mag.
        : (28/01/2016) integFuct improved.
        : (19/04/2017) SRPFITS added.
        : (16/11/2021) SRPSTATS added.
"""


import math, random, os, copy, sys
from scipy.optimize import fmin
from . import SRPUtil
import numpy
from SRPSTATS.AverIterSigmaClipp import AverIterSigmaClipp
from SRPFITS.Photometry.getBackground import getBackground
from SRPFITS.Photometry.MinMax import MinMax
from SRPFITS.Photometry.centerMoment import centerMoment
from SRP.SRPMath.findPercArea import findPercArea
from SRP.SRPMath.integFuct import integFuct




class MiPhotData:
    def __init__ (self, Id, x0, y0, max, mag, emag, saturation=55000, exptime=1.0, zp=(25.0,0.0), mjd=-99.0, etm=-99.0, sky=0.0, airm=(1.0,0.0), tmg=0.0, etmg=0.0, calsts=(1.0,0.0), cmt='N'):
        self.Id = Id
        self.XC = float(x0)
        self.YC = float(y0)
        self.MAX = float(max)
        if float(mag) < 90:
            self.Mag = float(mag)+2.5*math.log10(exptime)
        else:
            self.Mag = float(mag)
        self.eMag = float(emag)
        self.MagCal = self.Mag+float(zp[0])
        self.eMagCal = math.sqrt(self.eMag**2+float(zp[1])**2)
        self.Comment = cmt
#                if self.MAX < saturation and self.Mag < 90:
#                        self.Comment = 'Ok'
#                elif self.MAX >= saturation:
#                        self.Comment = 'S'
        self.MJD = float(mjd)
        self.Exptime = float(etm)
        self.Sky = float(sky)
        self.ZP = float(zp[0])
        self.eZP = float(zp[1])
        self.ChiZP = float(calsts[0])
        self.NZP = int(calsts[1])
        self.Airm = float(airm[0])
        self.Coef = float(airm[1])
        self.TMag = float(tmg)
        self.eTMag = float(etmg)
        self.DM = 0.0
        self.eDM = 0.0


    def __str__ (self):
        msg = ''
        msg = msg + "%10s\t%7.2f\t%7.2f\t%10.2f\t" % (self.Id, self.XC, self.YC, self.MAX)
        msg = msg + "%7.3f\t%7.3f\t%7.3f\t%7.3f\t" % (self.Mag, self.eMag, self.MagCal, self.eMagCal)
        msg = msg + "%4s\t%15.6f\t%10.1f\t%10.2f\t" % (self.Comment, self.MJD, self.Exptime, self.Sky)
        msg = msg + "%7.3f\t%7.3f\t%7.3f\t%6d\t%7.2f\t%7.3f\t" % (self.ZP, self.eZP, self.ChiZP, self.NZP, self.Airm, self.Coef)
        msg = msg + "%7.3f\t%7.3f" % (self.DM, self.eDM)
        return msg+os.linesep


    def __cmp__ (self, other):
        if self.MJD > other.MJD:
            return 1
        elif self.MJD == othr.MJD:
            return 0
        else:
            return -1




def findPercAreaBis (x,y,r,xp,yp):
    intersezioni = []
    xA = xp - 1
    sqr = r**2-(xA-x)**2
    if sqr >= 0:
        yA1 = y + math.sqrt(sqr)
        yA2 = y - math.sqrt(sqr)
        if yp-1 <= yA1 <= yp:
            intersezioni.append((xA,yA1,True,1))
        if yp-1 <= yA2 <= yp:
            intersezioni.append((xA,yA2,False,1))
    xB = xp
    sqr = r**2-(xB-x)**2
    if sqr >= 0:
        yB1 = y + math.sqrt(sqr)
        yB2 = y - math.sqrt(sqr)
        if yp-1 <= yB1 <= yp:
            intersezioni.append((xB,yB1,True,2))
        if yp-1 <= yB2 <= yp:
            intersezioni.append((xB,yB2,False,2))
    yC = yp - 1
    sqr = r**2-(yC-y)**2
    if sqr >= 0.0:
        xC1 = x + math.sqrt(sqr)
        xC2 = x - math.sqrt(sqr)
        if xp-1 <= xC1 <= xp:
            intersezioni.append((xC1,yC,True,3))
        if xp-1 <= xC2 <= xp:
            intersezioni.append((xC2,yC,False,3))
    yD = yp
    sqr = r**2-(yD-y)**2
    if sqr >= 0.0:
        xD1 = x + math.sqrt(sqr)
        xD2 = x - math.sqrt(sqr)
        if xp-1 <= xD1 <= xp:
            intersezioni.append((xD1,yD,True,4))
        if xp-1 <= xD2 <= xp:
            intersezioni.append((xD2,yD,False,4))
    if len(intersezioni) == 2:
#                area = (intersezioni[0][1]-(yp-1)+intersezioni[1][1]-(yp-1))/2.0
        if yp-1 >= y or yp <= y:
            if intersezioni[0][0] <= intersezioni[1][0]:
                area1 = (intersezioni[0][0]-(xp-1))*(intersezioni[0][1]-(yp-1))
                area3 = (xp-intersezioni[1][0])*(intersezioni[1][1]-(yp-1))
                if yp-1 >= y:
                    area2 = integFuct(intersezioni[0][0],intersezioni[1][0],x,y,r,yp)
                elif yp <= y:
                    area2 = integFuct(intersezioni[0][0],intersezioni[1][0],x,y,r,yp,False)
            else:
                area1 = (intersezioni[1][0]-(xp-1))*(intersezioni[1][1]-(yp-1))
                area3 = (xp-intersezioni[0][0])*(intersezioni[0][1]-(yp-1))
                if yp-1 >= y:
                    area2 = integFuct(intersezioni[1][0],intersezioni[0][0],x,y,r,yp)
                elif yp <= y:
                    area2 = integFuct(intersezioni[1][0],intersezioni[0][0],x,y,r,yp,False)
            area = area1 + area2 + area3
        else:
            if xp-1 >= x:
                if intersezioni[0][1] <= intersezioni[1][1]:
                    area = integFuct(intersezioni[0][1],intersezioni[1][1],y,x,r,xp)
                else:
                    area = integFuct(intersezioni[1][1],intersezioni[0][1],y,x,r,xp)
            else:
                if intersezioni[0][1] <= intersezioni[1][1]:
                    area = integFuct(intersezioni[0][1],intersezioni[1][1],y,x,r,xp,False)
                else:
                    area = integFuct(intersezioni[1][1],intersezioni[0][1],y,x,r,xp,False)
#                print area
#                print intersezioni
#                print
#                if intersezioni[0][3] in [1,2]:
#                        print Funct(intersezioni[0][0],x,y,r,intersezioni[0][2])
#                else:
#                        print Funct(intersezioni[0][1],y,x,r,intersezioni[0][2])
#                if intersezioni[1][3] in [1,2]:
#                        print Funct(intersezioni[1][0],x,y,r,intersezioni[1][2])
#                else:
#                        print Funct(intersezioni[1][1],y,x,r,intersezioni[1][2])
        if yp-1 >= y:
            return area
        elif yp <= y:
            return 1.0-area
        else:
            if xp-1 >= x:
                return area
            elif xp <= x:
                return 1-area
    else:
        dist = math.sqrt((x-xp)**2+(y-yp)**2)
        if dist < r:
            return 1.0
        else:
            return 0.0


def sumApert (table, x, y, r):
    if math.floor(x-r) < 1:
        xmin = 1
    else:
        xmin = int(math.floor(x-r))
    if math.ceil(x+r) > table.shape[1]:
        xmax = table.shape[1]
    else:
        xmax = int(math.ceil(x+r))
    if math.floor(y-r) < 1:
        ymin = 1
    else:
        ymin = int(math.floor(y-r))
    if math.ceil(y+r) > table.shape[0]:
        ymax = table.shape[0]
    else:
        ymax = int(math.ceil(y+r))
    field = table[(ymin-1):ymax,(xmin-1):xmax]
    tot = 0.0
#        npix = 0
    npixbis = 0
    maxf = 0.0
    for i in range(field.shape[1]):
        for l in range(field.shape[0]):
            dist = math.sqrt((x-(l+xmin-0.5))**2+(y-(i+ymin-0.5))**2)
            if dist <= r-0.5*math.sqrt(2):
                tot = tot + field[l,i]
#                                npix = npix + 1
                npixbis = npixbis + 1
                if field[l,i] > maxf:
                    cy,cx = i,l
                    maxf = field[l,i]
            elif r-0.5*math.sqrt(2) < dist <= r+0.5*math.sqrt(2):
#                                pa = findPercArea (x,y,r,l+xmin,i+ymin)
                pabis = findPercAreaBis (x,y,r,l+xmin,i+ymin)
                tot = tot + pabis*field[l,i]
#                                npix = npix + pa
                npixbis = npixbis + pabis
    #if math.fabs((math.pi*r**2-npixbis)/npixbis) > 1.0:
    #    sys.exit(1)
    return tot, npixbis, maxf






def computeMag (flux,back,ebg,npix,gain=1.0,ron=0.0,upsig=3):
    nflux = flux-npix*back
    enflux = math.sqrt(math.fabs(flux)*gain + npix*(ebg*gain)**2 + npix*ron**2)
#        print nflux,enflux,flux,back,ebg,npix
    if nflux > 0.0 and enflux > 0.0:
        emag = (2.5/math.log(10.0))*enflux/(nflux*gain)
        mag = -2.5*math.log10(nflux)
        if emag > 0.5:
            mag = -2.5*math.log10(upsig*enflux)
            emag = 99.0
    elif nflux <= 0.0 and enflux > 0.0:
        mag = -2.5*math.log10(upsig*enflux)
        emag = 99.0
    else:
        mag = 99
        emag = 99
    return mag,emag





def AverWeight(wlst,wei):
    if len(wlst) == 0:
        return None, None
    elif len(wlst) == 1:
        return wlst[0],wei[0]
    sum = 0.0
    wsum = 0.0
    for i in range(len(wlst)):
        sum = sum + wlst[i]*wei[i]**-2
        wsum = wsum + wei[i]**-2
#                print wlst[i],wei[i]
    err = 0.0
    ave = sum/wsum
    for i in range(len(wlst)):
        err = err + (ave-wlst[i])**2
    errf = math.sqrt(err/(len(wlst)-1))
#        print ave, errf/math.sqrt(len(wlst))
    return ave, errf/math.sqrt(len(wlst))



def averageZeroPoint (inplst, einplst, siglev = 2.0):
    wlst = copy.copy(inplst)
    ewlst = copy.copy(einplst)
    card = len(wlst)
    while True:
#                print wlst
#                print ewlst
        if len(wlst) < 1:
            return None,None
        elif len(wlst) == 1:
            return wlst[0],ewlst[0]
        elif len(wlst) == 2:
            return AverWeight(wlst,ewlst)
        elif 3 <= len(wlst) <= 10:
            wa = numpy.mean(wlst)
            dst = 0.0
            for i,l in zip(wlst,ewlst):
                dstil = math.fabs(wa-i)/l
                if dstil > dst:
                    dst = dstil
                    refi,refl = i,l
            if dst > 3:
                wlst.remove(refi)
                ewlst.remove(refl)
#                                print wa,refi,refl,dst
            else:
                return AverWeight(wlst,ewlst)
        else:
            wa = numpy.mean(wlst)
            ws = numpy.std(wlst,ddof=1)
            wmin, wmax = wa - siglev*ws, wa + siglev*ws
#                        print wa,ws,wmin,wmax
            for i,l in zip(wlst,ewlst):
                if not wmin <= i <= wmax:
                    wlst.remove(i)
                    ewlst.remove(l)
            if card != len(wlst):
                card = len(wlst)
            else:
                return AverWeight(wlst,ewlst)



def simplerAverageZeroPoint (inplst, einplst):
    wlst = copy.copy(inplst)
    ewlst = copy.copy(einplst)
    if len(wlst) < 1:
        return None,None
    elif len(wlst) == 1:
        return wlst[0],ewlst[0]
    else:
        wl = numpy.mean(wlst)
        ws = numpy.std(wlst,ddof=1)
        we = ws/math.sqrt(len(wlst))
        return wl,we






""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 16/07/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Correct the measures polarization in case of low S/N
            according to Plaszczynski et al., 2014, MNRAS 439, 4048.

History : (16/07/2014) First version.
"""

import numpy


def PolBias (p, ep):
    if ep <= 0:
        return p
    else:
        fact = 1.-numpy.exp(-(p/ep)**2)
        return (p - ep**2 * fact / (2*p))




""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/08/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (02/08/2014) First version.

"""

import warnings
from astroquery.simbad import Simbad
from SRP.SRPMath.AstroCoordInput import AstroCoordInput



class SimbadName:
    def __init__ (self, name):
        self.Name = name

        
        
    def GetData(self):
        d = Simbad.query_object(self.Name)
        return d
        

    def GetCoords(self):
        warnings.resetwarnings()
        warnings.filterwarnings('ignore', category=UserWarning, append=True)
        d = self.GetData()
        warnings.resetwarnings() 
        warnings.filterwarnings('always', category=UserWarning, append=True)
        #
        try:
            ra = d['RA'].data[0]
            dec = d['DEC'].data[0]
        except KeyError:
            return None
        except IndexError:
            return None
        #
        rap = ra.replace(' ',':')
        decp = dec.replace(' ',':')
        #
        return AstroCoordInput(rap,decp)
        

 


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 14/05/2019
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : 

Remarks :

History : (14/05/2019) First version.
"""

from astropy.table import Table
from astropy.coordinates import SkyCoord
from astropy.coordinates import search_around_sky
from astropy import units as u
from astroquery.vizier import Vizier
import warnings




def GetSDSS(rad,decd,rds):
    #
    crd = SkyCoord(ra=rad*u.degree, dec=decd*u.degree)
    #
    warnings.filterwarnings('ignore', category=UserWarning, append=True)
    try:
        restab = Vizier.query_region(crd, radius=rds*u.arcmin, catalog=["SDSS12"])
    except:
        restab = []
    warnings.resetwarnings()
    if len(restab) == 0:
        return []
    else:
        return restab[0]



""" Utility functions and classes for SRP

Context : SRP
Module  : Databases.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 29/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : 

Remarks :

History : (21/02/2017) First version.
        : (29/04/2017) 1mi timeout added.
"""

import mysql.connector as pm
from astropy.table import Table


def readMySQLTable (ihost, idbase, iuser, ipwd, icmd):
    db = pm.connect(user=iuser, password=ipwd, host=ihost, database=idbase, connect_timeout=60)
    cursor = db.cursor()
    cursor.execute(icmd)
    cols = cursor.column_names
    dt = [a for a in cursor]
    t = Table(rows=dt,names=cols)
    return t




""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 30/03/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians, real and imaginary part of the refraction index
            can be found at: http://refractiveindex.info/
            ambri e metri sono gli indici di rifrazione dell'ambiente e del mezzo.
            From Krijger et al., Atmos. Meas. Tech., 7, 3387–3398, 2014
            

History : (30/03/2017) First version.
"""

import numpy as np
import cmath



def rp (n1,n2, phi=np.radians(45.0)):
    CF = np.cos(phi)
    SF = np.sin(phi)
    RD = ((n1/n2)*SF)
    #
    N = n2*CF - n1*np.sqrt(1-RD**2)
    D = n2*CF + n1*np.sqrt(1-RD**2)
    return N/D
#
def rs (n1,n2, phi=np.radians(45.0)):
    CF = np.cos(phi)
    SF = np.sin(phi)
    RD = ((n1/n2)*SF)
    #
    N = n1*CF - n2*np.sqrt(1-RD**2)
    D = n1*CF + n2*np.sqrt(1-RD**2)
    return N/D
#
def MuellerMetallicMirrorMatrixAlter (ambri, metri, angle=np.radians(45.0)):
    RP = rp(ambri, metri, angle)
    RS = rs(ambri, metri, angle)
    DLT = cmath.phase(RP)-cmath.phase(RS)
    r1 = [0.5*(abs(RP)**2+abs(RS)**2), 0.5*(abs(RS)**2-abs(RP)**2), 0., 0.]
    r2 = [0.5*(abs(RS)**2-abs(RP)**2), 0.5*(abs(RP)**2+abs(RS)**2), 0., 0.]
    r3 = [0., 0., abs(RP)*abs(RS)*np.cos(DLT), abs(RP)*abs(RS)*np.sin(DLT)]
    r4 = [0., 0., -abs(RP)*abs(RS)*np.sin(DLT), abs(RP)*abs(RS)*np.cos(DLT)]
    return np.matrix([r1, r2, r3, r4])
#




""" Init file for SRPPolarimetry

Context : SRP
Module  : Polarimetry
Version : 1.6.0
Author  : Stefano Covino
Date    : 20/08/2018
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (21/02/2012) First named version.
        : (29/02/2012) Wave plate matrix added.
        : (02/09/2013) ISM matrix added.
        : (16/07/2014) PolBias added.
        : (20/01/2015) Simple transmission matrix added.
        : (22/05/2015) Polarizer matrix added.
        : (17/02/2017) Rayleigh sky model.
        : (30/03/2017) Alternative formulation of the metallic mirror matrix.
        : (09/05/2017) Better formulation of Mueller transmission matrix in case of beam splitting.
        : (18/09/2017) Cnt2Stoke.
        : (20/08/2018) Polarization limits based on Plaszczynski et al. (2014).
"""


__all__ = ['AluminiumRefractiveIndex','Cnt2Stoke', 'MuellerISMMatrix','MuellerMetallicMirrorMatrix',
           'MuellerMetallicMirrorMatrixAlter','MuellerRotationMatrix','MuellerTransmissionMatri',
           'MuellerHalfWavePlateMatrix','MuellerPolarizerMatrix', 'MuellerQuarterWavePlateMatrix',
           'MuellerThomsonRayleighScatteringMatrix','MuellerWavePlateMatrix','Pol2Stokes',
           'PolBias','PolBiasLims', 'RayleighSkyModel','Stokes2Pol','TransmissionMatrix']






""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/11/2015) First version.

"""


import numpy



def findPercArea (x,y,r,xp,yp,ntrial=10000):
    #
    xpp = numpy.random.uniform(xp-1,xp,ntrial)
    ypp = numpy.random.uniform(yp-1,yp,ntrial)
    #
    dist = numpy.sqrt((x-xpp)**2+(y-ypp)**2)
    l = dist[dist <= r]
    #
    return float(len(l))/ntrial





""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 14/02/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (14/02/2016) First version.
"""

import os
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from .GetVizierCat import GetVizierCat


class SkyBot:
    skybotaddress = "vo.imcce.fr"
    # pars epoch (JD),ra (deg), dec (deg), radius (arcsec), maxposerror (arcsec)
    skybotquery = "/webservices/skybot/skybotconesearch_query.php?-ep=%s&-ra=%.5f&-dec=%.5f&-rs=%.1f&-mime=text&-outout=object&-filter=%.1f&-from=SRP"
    # SkyBot entries
    
    
    
    class CatEntries:
        def __init__ (self,objid,ra,dec,objclass,vmag,poserr,cdist):
            self.Id = objid
            self.RA = float(ra)
            self.DEC = float(dec)
            self.Class = objclass
            inpmag = AstroMagInput (vmag,0)
            self.V = inpmag.Mag
            self.PosErr = poserr
            self.CDist = cdist
            
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%s\t%10.2f\t" % (self.Class, self.V)
            msg = msg + "%10.1f\t%10.1f" % (self.PosErr, self.CDist)
            return msg
            

        def __lt__ (self, other):
            return self.CDist < other.CDist


    
    
    def __init__ (self, ra, dec, radius=5.0, time=2000.0, maxerr=10):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=2000)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Epoch = time
        self.Radius = radius
        self.MaxError = maxerr
        self.ListEntries = []
        
        
    def GetData(self):
        QueryStr = self.skybotquery % (self.Epoch,self.RA,self.DEC,self.Radius,self.MaxError)
        data = GetVizierCat(self.skybotaddress,QueryStr)
        #
        if data != None:
            for i in data:
                il = i.split('|')
                if len(il) < 7:
                    return 0
                objid = il[1]
                ra = float(il[2])*15.0
                dec = float(il[3])
                coord = AstroCoordInput(ra,dec)
                objclass = il[4]
                objvmag = float(il[5])
                poserr = float(il[6])
                cdist = float(il[7])
                newinp = self.CatEntries(objid,coord.RA,coord.DEC,objclass,objvmag,poserr,cdist)
                self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg



    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tClass\tV\tPosErr\tCDist\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg

        
        
    def sort(self):
        self.ListEntries.sort()
        
        


""" Utility functions and classes for SRP

Context : SRP
Module  : Math
Version : 1.0.0
Author  : Stefano Covino
Date    : 18/11/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (18/11/2014) First version.

"""


import numpy



def LorentzFactor (Beta):
    Beta = 1./numpy.sqrt(1-Beta**2)
    return Beta






""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 26/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (03/09/2011) First version.
        : (26/05/2017) Minor update.
"""

import math
from SRP.SRPSpectroscopy.LymAlphaDumpConst import LymAlphaDumpConst
import SRP.SRPSpectroscopy as SS


def Ralpha (gu=3.0,gl=1.0):
    return (LymAlphaDumpConst() * SS.LambdaLymanAlpha) / (4 * math.pi * SS.SpeedofLight)




""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 10/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (10/11/2015) First version.

"""

import numpy

def BiGauss (data,x0,y0,A,sx,sy,B):
    x = numpy.arange(data.shape[1])
    xn = numpy.arange(data.shape[0])
    y = xn[:,numpy.newaxis]
    return A*numpy.exp( (-(x-x0)**2)/(2*sx**2) + (-(y-y0)**2)/(2*sy**2) ) + B


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscpy
Version : 1.1.0
Author  : Stefano Covino
Date    : 23/07/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : l (micron)

History : (22/08/2012) First version.
        : (23/07/2015) python3 porting.
"""


import os
from scipy.interpolate import interp1d
from SRP.SRPSystem.SRPPath import SRPPath
from astropy.table import Table



def AtmExtinction (l, fname):
    try:
        t = Table.read(os.path.join(SRPPath(),'SRPData','AtmExtinction',fname), format='ascii')
    except IOError:
        return None
    #
    a = interp1d(t['col1'], t['col2'], kind='quadratic', bounds_error=True)
    #
    la = l*1e4
    try:
        return a(la)
    except ValueError:
        return None




""" Init file for SRPMath

Context : SRP
Module  : Math
Version : 1.6.0
Author  : Stefano Covino
Date    : 14/03/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (29/09/2010) First named version.
        : (08/11/2010) Cartesian distance added.
        : (09/02/2012) Sidereal time, hour and parallactic angles computation added.
        : (11/02/2012) PointMatch added.
        : (18/02/2012) Sky functions moved to SRPSky.
        : (23/08/2012) New angular distance function.
        : (20/09/2012) Angular phasing function.
        : (18/11/2014) Doppler factor function.
        : (10/11/2015) BiGauss function.
        : (18/11/2015) raw_moment function.
        : (27/11/2015) findPercArea function.
        : (28/01/2016) integFuct function.
        : (14/03/2017) AngleRange function.
"""


__all__ = ['AngleRange','AngularDistance', 'AstroAngleInput', 'AstroCoordInput', 'AstroMagInput',
           'BiGauss','CartesiamCoordAmpRotoTranslation','CartesianDistance', 'CartesianVectorClass',
           'DopplerFactor','FastAngularDistance', 'findPercArea','inetgFuct',
           'InverseCartesiamCoordAmpRotoTranslation','LorentzFactor','MinDist', 'PhaseAngle',
           'PointClass', 'PointMatch', 'raw_moment','TriangleClass']





""" Utility functions and classes for SRP

Context : SRP
Module  : Databases.py
Version : 1.2.1
Author  : Stefano Covino
Date    : 29/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : 

Remarks :

History : (21/02/2017) First version.
        : (13/03/2017) Wrtten symmetrically with the MySQL version.
        : (14/03/2017) Rewritten in a more efficient way for the given library.
        : (29/04/2017) 1min timeout added.
"""

import postgresql as ps
from astropy.table import Table


def readPostgresTable (ihost, idbase, iuser, ipwd, icmd):
    db = ps.open(host=ihost,database=idbase,user=iuser,password=ipwd,connect_timeout=60)
    cursor = db.prepare(icmd)
    cols = cursor.column_names
    dt = [a for a in cursor]
    t = Table(rows=dt,names=cols)
    return t




""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 14/05/2019
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (16/02/2016) First version.
        : (14/05/2019) Minor upgrade.
"""

import os
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from astropy.coordinates import SkyCoord
from astropy.coordinates import search_around_sky
from astropy import units as u
from astroquery.vizier import Vizier
from astroquery.vizier import Conf
import warnings


class IGAIA:
    Conf.row_limit = -1
    #  
    class CatEntries:
        def __init__ (self,objid,ra,dec,mb,emb,mr,emr,mg,emg):
            self.Id = objid
            self.RA = float(ra)
            self.DEC = float(dec)
            inpmag = AstroMagInput (mb,emb)
            self.B = inpmag.Mag
            self.eB = inpmag.eMag
            inpmag = AstroMagInput (mr,emr)
            self.R = inpmag.Mag
            self.eR = inpmag.eMag
            inpmag = AstroMagInput (mg,emg)
            self.G = inpmag.Mag
            self.eG = inpmag.eMag
        
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%7.2f\t%7.2f\t" % (self.B, self.eB)
            msg = msg + "%7.2f\t%7.2f\t" % (self.R, self.eR)
            msg = msg + "%7.2f\t%7.2f" % (self.G, self.eG)
            return msg
            

        def __lt__ (self, other):
            return self.R < other.R

    
    
    def __init__ (self, ra, dec, radius=5.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=2000)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Radius = radius
        self.ListEntries = []
        
        
    def GetData(self):
        crd = SkyCoord(ra=self.RA*u.degree, dec=self.DEC*u.degree)
        warnings.filterwarnings('ignore', category=UserWarning, append=True)
        try:
            restab = Vizier.query_region(crd, radius=self.Radius*u.arcmin, catalog=["I/324/igsl3"])
        except:
            restab = []
        warnings.resetwarnings()
        #
        if len(restab) > 0:
            data = restab[0]
            for i in data:
                objid = 'IGAIA'
                ra = i['RAJ2000']
                dec = i['DEJ2000']
                coord = AstroCoordInput(ra,dec)
                mb = i['magBJ']
                emb = i['e_magBJ']
                mr = i['magRF']
                emr = i['e_magRF']
                mg = i['magG']
                emg = i['e_magG']
                newinp = self.CatEntries(objid,coord.RA,coord.DEC,mb,emb,mr,emr,mg,emg)
                self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg



    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tB\teB\tR\teR\tG\teG\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg

        
        
    def sort(self):
        self.ListEntries.sort()
        
        



""" Utility functions and classes for SRP

Context : SRP
Module  : Math
Version : 1.0.1
Author  : Stefano Covino
Date    : 22/03/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (14/03/2017) First version.
        : (22/03/2017) Better version.
"""


def AngleRange (ang):
    return ang % 360.0





""" Utility functions and classes for SRP

Context : SRP
Module  : Math
Version : 2.0.1
Author  : Stefano Covino
Date    : 12/11/2021
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (20/09/2012) First version.
        : (30/07/2020) Alternative version for [0,180] interval.
        : (12/11/2021) Both versions available.
"""

import numpy as np


def PhaseAngle (ang, mina=0.0, maxa=360.0):
    while ang < mina or ang >= maxa:
        if ang < mina:
            ang = ang + (maxa-mina)
        else:
            ang = ang - (maxa-mina)
    return ang



def phaseang (ang):
    x = np.cos(ang)
    y = np.sin(ang)
    return np.arctan(y/x)




""" 

Context : SRP
Module  : SRPDatabases
Version : 1.0.0
Author  : Stefano Covino
Date    : 21/02/2017
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (22/02/2017) First named version.
"""



__all__ = ['readMySQLTable', 'readPostgresTable']





""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 18/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (18/11/2015) First version.

"""

import numpy

def raw_moment(data, iord, jord):
    nrows, ncols = data.shape
    y, x = numpy.mgrid[:nrows, :ncols]
    data = data * x**iord * y**jord
    return data.sum()



""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 20/05/2021
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (20/05/2021) First version.

"""


import os, os.path


def WriteTextFile(filename,msg,mode='w'):
    with open(filename,mode) as f:
        f.write(msg)




""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 28/01/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (28/01/2016) First version.

"""


import numpy



def integFuct (a,b,x0,y0,r,y,plus=True):
    #        print a,b,x0,y0,r,y
    fct1 = y0*(b-a)
    fct2 = 0.5*r**2 * (numpy.arcsin((b-x0)/r) - numpy.arcsin((a-x0)/r))
    fct3 = 0.5*( (b-x0)*numpy.sqrt(r**2-(b-x0)**2) - (a-x0)*numpy.sqrt(r**2-(a-x0)**2))
    subt = (b-a)*(y-1)
    #        print fct1,fct2,fct3
    if plus:
        return fct1+fct2+fct3-subt
    else:
        return fct1-fct2-fct3-subt





""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 09/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : from Ilyn (2017)
        : relative trasmitance for the two beams F = B/A

History : (09/05/2017) First version.

"""

import numpy as np


def MuellerTransmissionMatrix (T1=1.,T2=0.,ebeam=True):
    #
    #
    A = (T1 + T2)/2.
    #
    B = (T1 - T2)/2.
    #
    C = np.sqrt(T1*T2)
    #
    if ebeam:
        Bt = B
    else:
        Bt = -B
    #
    r1 = [A, Bt, 0., 0.]
    r2 = [Bt, A, 0., 0.]
    r3 = [0., 0., C, 0.]
    r4 = [0., 0., 0., C]
    #
    return np.matrix([r1, r2, r3, r4])



""" Utility functions and classes for SRP

Context : SRP
Module  : Plot
Version : 1.0.0
Author  : Stefano Covino
Date    : 12/08/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : SetPlotPars(...)
        : px: subplot (e.g. px = p.add_subplot(111))

Remarks :

History : (12/08/2020) First version.

"""

import matplotlib.pyplot as plt


def SetPlotPars (px,xlbl="",ylbl="",titlbl="",linwidth=2,tickxfonsiz=24,tickyfonsiz=24,xlblfonsiz=30,ylblfonsiz=30,titlblfonsiz=35):
    #
    # Lines
    [i.set_linewidth(linwidth) for i in px.spines.values()]
    #
    # Ticks
    plt.tick_params(axis='both', which='major', labelsize='xx-large')
    plt.tick_params(axis='both', which='minor', labelsize='x-large')
    #
    for tickx,ticky in zip(px.xaxis.get_major_ticks(),px.yaxis.get_major_ticks()):
        tickx.label.set_fontsize(tickxfonsiz) 
        ticky.label.set_fontsize(tickyfonsiz)
    #
    # Labels
    plt.xlabel(xlbl,fontsize=xlblfonsiz)
    plt.ylabel(ylbl,fontsize=ylblfonsiz)
    plt.title(titlbl,fontsize=titlblfonsiz)
#






""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.0.1
Author  : Stefano Covino
Date    : 08/11/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (15/02/2012) First version.
        : (08/11/2017) Better coding.
"""


import numpy


def Mag2Counts (mag, emag, zp=0.0):
    cnt = 10**(-0.4*(numpy.array(mag)-zp))
    ecnt = (numpy.array(emag)*cnt)/(2.5/numpy.log(10.0))
    return cnt, ecnt



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 31/05/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (11/08/2010) First version.
        : (31/05/2021) Beter management of warnings.
"""


import ephem
import math
import warnings


from . import MathConstants




class AstroMagInput:
    def __init__ (self, mag, emag):
        #
        warnings.filterwarnings('error', category=UserWarning, append=True)
        #
        try:
            self.Mag = float(mag)
        except (ValueError, UserWarning):
            self.Mag = MathConstants.BADMAG
        try:
            self.eMag = float(emag)
        except (ValueError, UserWarning):
            self.eMag = MathConstants.BADMAG 
        #
        warnings.resetwarnings()
        
        
    def __str__ (self):
        msg = "%10.3f %9.3f" % (self.Mag, self.eMag)
        return msg


""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 23/07/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (05/10/2010) First version.
        : (15/10/2010) Wrong output for SDSS filters.
        : (16/10/2010) Output better suited for skycat.
        : (05/02/2012) Better path management.
        : (07/01/2013) Better formatting for output.
        : (23/07/2015) python3 comparison.

"""

import os, os.path

import atpy, numpy

import SRP
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPMath.AngularDistance import AngularDistance
from SRP.SRPSystem.SRPPath import SRPPath



class AstroWiseStandardStars:
    basepath = SRPPath()
    filename = 'cal569E_v11.fits'
    catpath = os.path.join(basepath,'SRPData','Catalogues','AstroWiseStandardStars',filename)
    # Catalogue entries
    class CatEntries:
        def __init__ (self, xxx_todo_changeme):
            # Id
            (id,ra,dec,u,eu,b,eb,v,ev,r,er,i,ei,su,esu,sg,esg,sr,esr,si,esi,sz,esz) = xxx_todo_changeme
            self.Id = id
            # Coord
            coord = AstroCoordInput(ra,dec)
            self.RA = coord.RA
            self.DEC = coord.DEC
            # U mag
            inpmag = AstroMagInput (u,eu)
            self.U = inpmag.Mag
            self.eU =inpmag.eMag
            # B mag
            inpmag = AstroMagInput (b,eb)
            self.B = inpmag.Mag
            self.eB =inpmag.eMag
            # V mag
            inpmag = AstroMagInput (v,ev)
            self.V = inpmag.Mag
            self.eV =inpmag.eMag
            # R mag
            inpmag = AstroMagInput (r,er)
            self.R = inpmag.Mag
            self.eR =inpmag.eMag
            # I mag
            inpmag = AstroMagInput (i,ei)
            self.I = inpmag.Mag
            self.eI =inpmag.eMag
            # u mag
            inpmag = AstroMagInput (su,esu)
            self.u = inpmag.Mag
            self.eu =inpmag.eMag
            # g mag
            inpmag = AstroMagInput (sg,esg)
            self.g = inpmag.Mag
            self.eg =inpmag.eMag
            # r mag
            inpmag = AstroMagInput (sr,esr)
            self.r = inpmag.Mag
            self.er =inpmag.eMag
            # i mag
            inpmag = AstroMagInput (si,esi)
            self.i = inpmag.Mag
            self.ei =inpmag.eMag
            # z mag
            inpmag = AstroMagInput (sz,esz)
            self.z = inpmag.Mag
            self.ez =inpmag.eMag
            # sexcoord
            self.Coord = coord

            
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.U, self.eU)
            msg = msg + "%10.3f\t%9.3f\t" % (self.B, self.eB)
            msg = msg + "%10.3f\t%9.3f\t" % (self.V, self.eV)
            msg = msg + "%10.3f\t%9.3f\t" % (self.R, self.eR)
            msg = msg + "%10.3f\t%9.3f\t" % (self.I, self.eI)            
            msg = msg + "%10.3f\t%9.3f\t" % (self.u, self.eu)
            msg = msg + "%10.3f\t%9.3f\t" % (self.g, self.eg)
            msg = msg + "%10.3f\t%9.3f\t" % (self.r, self.er)
            msg = msg + "%10.3f\t%9.3f\t" % (self.i, self.ei)
            msg = msg + "%10.3f\t%9.3f" % (self.z, self.ez)
            msgo = "".join(i for i in msg if 0 < ord(i) < 128)
            return "\t".join(msgo.split())
            


        def __lt__ (self, other):
            return self.V < other.V


    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Radius = radius/60.0
        self.ListEntries = []
        
        
    def GetData(self):
        tb = atpy.Table(self.catpath,type='fits')
        for i in range(len(tb)):
            if AngularDistance((self.RA,self.DEC),(tb.Ra[i],tb.Dec[i])) <= self.Radius:
                datum = self.CatEntries((tb.Name[i],tb.Ra[i],tb.Dec[i],tb.JohnsonU[i],tb.JohnsonU_err[i],
                        tb.JohnsonB[i],tb.JohnsonB_err[i],tb.JohnsonV[i],tb.JohnsonV_err[i],
                        tb.CousinsR[i],tb.CousinsR_err[i],tb.CousinsI[i],tb.CousinsI_err[i],
                        tb.SloanU[i],tb.SloanU_err[i],tb.SloanG[i],tb.SloanG_err[i],
                        tb.SloanR[i],tb.SloanR_err[i],tb.SloanI[i],tb.SloanI_err[i],
                        tb.SloanZ[i],tb.SloanZ_err[i]))
                self.ListEntries.append(datum)
        return len(self.ListEntries)

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg


    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tJohnsonU\teJohnsonU\tJohnsonB\teJohnsonB\tJohnsonV\teJohnsonV\tCousinsR\teCousinsR\t"
        msg =  msg + "CousinsI\teCousinsI\tSloanU\teSloanU\tSloanG\teSloanG\tSloanR\teSloanR\tSloanI\teSloanI\tSloanZ\teSloanZ\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg
        
        
    def sort(self):
        self.ListEntries.sort()


""" 

Context : SRP
Module  : Tables
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/10/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/10/2015) First version.

"""



def ReadCleanedTextFile(fname):
    with open(fname) as f:
        fdata = f.readlines()
    #
    listadata = []
    for i in fdata:
        if len(i.strip()) > 0:
            listadata.append(i.strip())
    #
    return listadata


""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 20/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Compute the polarisation limits in case of low S/N
            according to Plaszczynski et al., 2014, MNRAS 439, 4048.

History : (20/08/2018) First version.
"""

import numpy as np


def PolBiasLims (p, conf='68'):
    if conf == '68':
        pa = 1.0
        betam = 0.72
        gammam = 0.60
        omegam = -0.83
        phim = 4.41
        #
        betaM = 0.97
        gammaM = 2.01
    elif conf=='90':
        pa = 1.64
        betam = 0.88
        gammam = 0.68
        omegam = 2.03
        phim = -0.76
        #
        betaM = 0.31
        gammaM = 2.25
    elif conf=='95':
        pa = 1.95
        betam = 0.56
        gammam = 0.48
        omegam = 1.79
        phim = -1.03
        #
        betaM = 0.22
        gammaM = 2.54
    #
    pmin = p - pa*(1+betam*np.exp(-gammam*p)*np.sin(omegam*p+phim))
    pmax = p + pa*(1-betaM*np.exp(-gammaM*p))
    #
    return pmin,pmax






""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (05/09/2011) First version.

"""


from SRP.SRPSpectroscopy.LyAlphaProfile import LyAlphaProfile


def TauDLAVoigt (lamb,NHI,bpar,zDLA):
    res = NHI * LyAlphaProfile( (lamb/(1.0+zDLA)), bpar)
    return res




""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians
            

History : (22/05/2015) First version.

"""

import math
import numpy


def MuellerPolarizerMatrix (angle, eff=1.0):
    C = numpy.cos(2*angle)
    S = numpy.sin(2*angle)
    r1 = [1., eff*C, eff*S, 0.]
    r2 = [eff*C, eff*C**2, eff*C*S, 0.]
    r3 = [eff*S, eff*C*S, eff*S**2, 0.]
    r4 = [0., 0., 0., 0.]
    return 0.5*numpy.matrix([r1, r2, r3, r4])



from setuptools import setup, find_packages
from os import path
import glob
import SRP
import os

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


# Look for script files
lscr = glob.glob(os.path.join('Scripts', 'SRP*'))
lscrex = []
for i in lscr:
    if os.path.splitext(i)[1] == '':
        lscrex.append(i)

# Look for data files
lsdt = glob.glob(os.path.join('SRP/SRPData', '*'))
lsdtex = []
for i in lsdt:
    if os.path.splitext(i)[1] == '':
        lsdtex.append(i)


setup(
    name='SRPAstro',
    version=SRP.__version__,
    description='Astronomical Data Analysis Tools',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://pypi.python.org/pypi/SRPAstro',
    author='Stefano Covino',
    author_email='stefano.covino@inaf.it',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Astronomy',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3',
    ],
    keywords='astronomy data analysis',
    packages=find_packages(),
    python_requires='>=3',
    include_package_data=True,
    package_data={'SRP':lsdtex},
    scripts=lscrex,
    install_requires=['mysql-connector-python', 'py-postgresql', 'requests', 'astroquery',
        'astropy>=0.4', 'scipy', 'astlib>=0.4', 'matplotlib', 'atpy', 'asciitable', 'ephem',
        'numpy>=1.1', 'dust_extinction'],
    )



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 25/02/2022
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported
        : lambda in micron units

History : (25/02/2022) First version.
"""

from dust_extinction.averages import G03_SMCBar
from dust_extinction.averages import G03_LMCAvg
from dust_extinction.averages import GCC09_MWAvg
import astropy.units as u
from . import ExtConstants
import numpy as np


extsmc = G03_SMCBar()
extlmc = G03_LMCAvg()
extmw = GCC09_MWAvg()

def getExt (ebv,glxy,lmbd):
    if not glxy in (list(ExtConstants.ExtCurDict.keys())):
        return None
    else:
        if glxy == 'SMC':
            absr = extsmc.extinguish(lmbd*u.micron,Ebv=ebv)
        elif glxy == 'LMC':
            absr = extlmc.extinguish(lmbd*u.micron,Ebv=ebv)
        elif glxy == 'MW':
            absr = extmw.extinguish(lmbd*u.micron,Ebv=ebv)
    return -2.5*np.log10(absr)




""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (03/09/2011) First version.

"""

import math
import SRP.SRPSpectroscopy as SS


def StndCriticalDens (H0=71.0):
    return (3 * (H0*0.1/SS.Parsec)**2) / (8 * math.pi * SS.GravitationalConst)



""" Init file for SRPPlot

Context : SRP
Module  : Plot
Version : 1.0.0
Author  : Stefano Covino
Date    : 12/08/2020
E-mail  : stefano.covino@inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (12/08/2020) First named version.
"""


__all__ = ['SetPlotPars', ]



